//////////////////////////////////////////////////////////////////////////////////////////
// Class MiniIMU9Accel - Defines an interface to the LSM303DLM accelerometer and compass
//                       on the Pololu MiniIMU9 board.  The IMU does not break out all
//                       functionality from the chip so not all registers from the device
//                       are implemented with functions.
//
//
// Copyright Kyle Crane 2012.  See Bottom for License Terms
////////////////////////////////////////////////////////////////////////////////////////// 

//////////////////////////////////////////////////////////////////////////////////////////
//      
// Public Members
//////////////////////////////////////////////////////////////////////////////////////////

#include "MiniIMU9Accel.h"

MiniIMU9Accel::MiniIMU9Accel()
{
    m_bus = 0;
    m_addr = 0;
}


MiniIMU9Accel::~MiniIMU9Accel()
{
    m_bus = 0;
    m_addr = 0;
}


void MiniIMU9Accel::Init(I2C& bus)
{
    Init(bus, MIMU_ACCEL_ADR);
}


void MiniIMU9Accel::Init(I2C& bus, uint8_t addr)
{
    m_bus = &bus;
    m_addr = addr;

    //SetByteOrder(1);
    //SetUpdateMode(1);
}


int MiniIMU9Accel::IsOnline()
{
    if(m_bus)
        return m_bus->DevPresent(m_addr);
    else
        return 0;
}


int MiniIMU9Accel::WriteReg(uint8_t reg, uint8_t value)
{
    if(m_bus)
    {
        if (reg >= 0x20 && reg <= 0x2D)
            return m_bus->TXByte(m_addr, reg, 1, value);
        else
            return -1;
    }
    else
        return -1;
}


uint8_t MiniIMU9Accel::ReadReg(uint8_t reg)
{
    if(m_bus)
    {
        if (reg >= 0x20 && reg <= 0x2D)
            return m_bus->RXByte(m_addr, reg, 1);
        else
            return -1;
    }
    else
        return -1;
}


int16_t MiniIMU9Accel::ReadChannel(uint8_t chan)
{
    if(m_bus)
    {
        int16_t chanVal = 0;
        uint8_t reg     = A_OUT_X_L + (chan << 1);
        reg |= A_AUTO_INC_M;

        chanVal = m_bus->RXWord(m_addr, reg, 1);
        chanVal >>= 4;

        return chanVal;
    }
    else
        return -1;
}


int    MiniIMU9Accel::ReadAllChannels(int16_t* buffer)
{
    if(m_bus)
    {
        uint8_t reg = A_OUT_X_L | A_AUTO_INC_M;
        int        result = -1;
        uint8_t temp[6];

        result = m_bus->RXBuf(m_addr, temp, 6, reg, 1);

        int n=0;
        for(int i=0; i<3; i++)
        {
            buffer[i] = (temp[n] << 8) | temp[n+1];
            buffer[i] >>= 4;
            n += 2;
        }

        return result;
    }
    else
        return -1;
}


int MiniIMU9Accel::Enable(uint8_t state)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(A_CTRL_REG1);

        if (state < 3)
        {
            state <<= 5;                 // Move state to correct bit position
            regVal &= ~A_POWER_MODE_M;   // Clear current bits
            regVal |= state;
            result = WriteReg(A_CTRL_REG1, regVal);
        }
        else
          result = -1;

        return result;
    }
    else
        return -1;
}


int MiniIMU9Accel::EnableAxis(uint8_t x, uint8_t y, uint8_t z)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t bitVal = ((z<<2) | (y<<1) | x) & 0x03;
        uint8_t regVal = ReadReg(A_CTRL_REG1);

        regVal &= ~A_AXIS_EN_M;
        regVal |= bitVal;

        return WriteReg(A_CTRL_REG1, regVal);
    }
    else
        return -1;
}


int MiniIMU9Accel::SetFullScale(uint8_t scale)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(A_CTRL_REG4);

        if (scale < 4)
        {
            scale <<= 4;                 // Move rate to correct bit position
            regVal &= ~A_FULL_SCALE_M;  // Clear current bits
            regVal |= scale;             // Set new bits
            result = WriteReg(A_CTRL_REG4, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}


int MiniIMU9Accel::SetDataRate(uint8_t rate)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(A_CTRL_REG1);

        if (rate < 4)
        {
            rate <<= 3;                  // Move rate to correct bit position
            regVal &= ~A_DATA_RATE_M;    // Clear current bits
            regVal |= rate;              // Set new bits
            result = WriteReg(A_CTRL_REG1, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}


int MiniIMU9Accel::SetFilter(uint8_t en, uint8_t cutoff)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(A_CTRL_REG2);

        if (en && cutoff < 4)
        {
            en = 1 << 4;                                // Move to correct bit position
            regVal &= ~(A_FILTER_M | A_FILTER_CO_M);    // Clear current bits
            regVal |= (en | cutoff);                    // Set new bits
            result = WriteReg(A_CTRL_REG2, regVal);
        }
        else if (cutoff < 4)
        {
            regVal &= ~(A_FILTER_M | A_FILTER_CO_M);    // Clear current bits
            regVal |= (cutoff);                         // Set new bits
            result = WriteReg(A_CTRL_REG2, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}




//////////////////////////////////////////////////////////////////////////////////////////
// Class MiniIMU9Accel
//      Private Members
//
//////////////////////////////////////////////////////////////////////////////////////////


int MiniIMU9Accel::SetByteOrder(uint8_t order)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(A_CTRL_REG4);

        if (order < 2)
        {
            order <<= 6;                 // Move rate to correct bit position
            regVal &= ~A_DATA_ORDER_M;   // Clear current bits
            regVal |= order;             // Set new bits
            result = WriteReg(A_CTRL_REG4, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}

int MiniIMU9Accel::SetUpdateMode(uint8_t type)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(A_CTRL_REG4);

        if (type < 2)
        {
            type <<= 7;                 // Move rate to correct bit position
            regVal &= ~A_UPDATE_MODE_M; // Clear current bits
            regVal |= type;             // Set new bits
            result = WriteReg(A_CTRL_REG4, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}

/*
Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.
*/
