//////////////////////////////////////////////////////////////////////////////////////////
// Class MiniIMU9Accel - Defines an interface to the LSM303DLM accelerometer and compass
//                       on the Pololu MiniIMU9 board.  The IMU does not break out all
//                       functionality from the chip so not all registers from the device
//                       are implemented with functions.
//
//
// Copyright Kyle Crane 2012.  See Bottom for License Terms
//////////////////////////////////////////////////////////////////////////////////////////

#ifndef MINI_IMU_9_ACCEL_H_
#define MINI_IMU_9_ACCEL_H_

#include <propeller.h>
#include "I2CBus.h"

enum ACCEL_CONSTANTS
{
    // Accelerometer register bit masks 
    MIMU_ACCEL_ADR  = 0x18,
    A_AUTO_INC_M    = 0x80,
    A_DATA_RATE_M   = 0x18,
    A_FILTER_M      = 0x10,
    A_FILTER_CO_M   = 0x03,
    A_POWER_MODE_M  = 0xE0,
    A_AXIS_EN_M		= 0x03, 
    A_UPDATE_MODE_M = 0x80,
    A_DATA_ORDER_M  = 0x40,
    A_FULL_SCALE_M  = 0x30, 
    
    // Accelerometer Register Addresses 
    A_CTRL_REG1             = 0x20,
    A_CTRL_REG2             = 0x21,
    A_CTRL_REG3             = 0x22,
    A_CTRL_REG4             = 0x23,
    A_CTRL_REG5             = 0x24,
    A_CTRL_REG6             = 0x25,
    A_HP_FILTER_RESET_REG   = 0x25, //DLHC Only
    A_REFERENCE_REG         = 0x26, //DLHC Only
    A_STATUS_REG            = 0x27,
    A_OUT_X_L               = 0x28, 
    A_OUT_X_H               = 0x29, 
    A_OUT_Y_L               = 0x2A, 
    A_OUT_Y_H               = 0x2B,
    A_OUT_Z_L               = 0x2C,    
    A_OUT_Z_H               = 0x2D
}; 

  
class MiniIMU9Accel
{
private:
	int16_t     m_data[3];							// Data array used by MiniIMU9Core if active.
	uint8_t     m_addr;								// Device address for the Accelerometer.
	I2C*        m_bus;								// Pointer to I2C Bus object to manage serial communication.

public:
	MiniIMU9Accel();
	~MiniIMU9Accel();
	void Init(I2C& bus);
	void Init(I2C& bus, uint8_t addr);


	int IsOnline();                                 // Returns: 1=Online 0=Offline.
	int WriteReg(uint8_t reg, uint8_t value);       // Write the value to specified Register. (-1 Error)
	uint8_t ReadReg(uint8_t reg);                   // Returns the value stores at the specified register.
	int16_t ReadChannel(uint8_t chan);              // Returns the current value from the accel channel.
	int	ReadAllChannels(int16_t* buffer);			// Read all 3 channels from the register data in a single bus transaction. (-1 Error)
	int16_t GetChannel(uint8_t chan);               // Returns the values provided from MiniIMU9Core if active.



	int Enable(uint8_t state);                      // Enable device and set operating mode 0 - 3.  (-1 Error)
	int EnableAxis(uint8_t x, uint8_t y, uint8_t z); // Enable or disable individual axis.  (-1 Error)
	int SetFullScale(uint8_t scale);                // Set G scale 0(2G) - 3(8G).  (-1 Error)
	int SetDataRate(uint8_t rate);                  // Set Data Rate 0(50Hz) - 3(1KHz).  (-1 Error)
	int SetFilter(uint8_t en, uint8_t cutoff);      // Setup HighPass Filter.  (-1 Error)
	int SetByteOrder(uint8_t order);                // Set register order 0(LSB First) - 1(MSB First)   (-1 Error)
	int SetUpdateMode(uint8_t type);                // Set output update mode 0(Continuous) - 1(Atomic)  (-1 Error)
};

#endif


/*
Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.
*/
