////////////////////////////////////////////////////////////////////////////////////////////
// Class MiniIMU9Compass - Defines an interface to the LSM303DLM accelerometer and compass
//                         on the Pololu MiniIMU9 board.  The IMU does not break out all
//                         functionality from the chip so not all registers from the device
//                         are implemented with functions.
//
//
// Copyright Kyle Crane 2012.  See Bottom for License Terms
////////////////////////////////////////////////////////////////////////////////////////////

#include "MiniIMU9Compass.h"

MiniIMU9Compass::MiniIMU9Compass()
{
	m_bus  = 0;
	m_addr = 0;
}


MiniIMU9Compass::~MiniIMU9Compass()
{
	m_bus  = 0;
	m_addr = 0;
}


void MiniIMU9Compass::Init(I2C& bus)
{
	Init(bus, MIMU_MAG_ADR);
}


void MiniIMU9Compass::Init(I2C& bus, uint8_t addr)
{
	m_bus = &bus;
    m_addr = addr;
}


int MiniIMU9Compass::IsOnline()
{
	if(m_bus)
		return m_bus->DevPresent(m_addr);
	else
		return -1;
}


int  MiniIMU9Compass::WriteReg(uint8_t reg, uint8_t value)
{
	if(m_bus)
	{
		if (reg >= M_CRA_REG && reg <= M_IRC_REG)
			return m_bus->TXByte(m_addr, reg, 1, value);
		else
			return -1;
	}
	else
		return -1;
}


uint8_t  MiniIMU9Compass::ReadReg(uint8_t reg)
{
	if(m_bus)
	{
		if (reg >= M_CRA_REG && reg <= M_IRC_REG)
			return m_bus->RXByte(m_addr, reg, 1);
		else
			return -1;
	}
	else
		return -1;
}


int16_t MiniIMU9Compass::ReadChannel(uint8_t chan)
{
	if(m_bus)
	{
		int16_t chanVal = 0;
		uint8_t reg = 0;

		if (chan == 0)
			reg = M_OUT_X_H;
		else if (chan == 1)
			reg = M_OUT_Y_H;
		else if (chan == 2)
			reg = M_OUT_Z_H;
		else
			return -1;

		chanVal = m_bus->RXWord(m_addr, reg, 1);

		return chanVal;
	}
	else
		return -1;
}


int MiniIMU9Compass::ReadAllChannels(int16_t* buffer)
{
	if(m_bus)
	{
		uint8_t reg = M_OUT_X_H;
		uint8_t temp[6];
		int		result = -1;

		result = m_bus->RXBuf(m_addr, temp, 6, reg, 1);

		buffer[0] = (temp[0]<<8) | temp[1];
		buffer[2] = (temp[2]<<8) | temp[3];
		buffer[1] = (temp[4]<<8) | temp[5];

		return result;
	}
	else
		return -1;
}


int MiniIMU9Compass::Enable(uint8_t value)
{
	if(m_bus)
	{
		int result = 0;
		uint8_t regVal = ReadReg(M_MR_REG);

		if(value)
			--value;

		if (value < 3)
		{
			regVal &= ~M_OP_MODE_M;   		// Clear current bits
			regVal |= value;	      		// Set new bits
			result = WriteReg(M_MR_REG, regVal);
		}
		else
		  result = -1;

		return result;
	}
	else
		return -1;
}


int MiniIMU9Compass::SetGain(uint8_t value)
{
	if(m_bus)
	{
		int result = 0;
		uint8_t regVal = ReadReg(M_CRB_REG);

		if (value < 8)
		{
			value <<= 5;				// Shift into position
		    regVal &= ~M_GAIN_M;   		// Clear current bits
		    regVal |= value;			// Set new bits
		    result = WriteReg(M_CRB_REG, regVal);
		}
		else
		  result = -1;

		return result;
	}
	else
		return -1;
}


int MiniIMU9Compass::SetDataRate(uint8_t value)
{
	if(m_bus)
	{
		int result = 0;
		uint8_t regVal = ReadReg(M_CRA_REG);

		if (value < 8)
		{
			value <<= 2;				// Shift into position
			regVal &= ~M_DATA_RATE_M;	// Clear current bits
			regVal |= value;			// Set new bits
			result = WriteReg(M_CRA_REG, regVal);
		}
		else
		  result = -1;

		return result;
	}
	else
		return -1;
}



/*
Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.
*/
